@@ -24,12 +24,13 @@
 #include <ctype.h>
 
 #include "ModemManager.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 #include "mm-errors-types.h"
 #include "mm-modem-helpers.h"
 #include "mm-base-modem-at.h"
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-3gpp.h"
+#include "mm-iface-modem-location.h"
 #include "mm-broadband-modem-telit.h"
 #include "mm-modem-helpers-telit.h"
 #include "mm-telit-enums-types.h"
@@ -38,14 +39,17 @@
 static void iface_modem_init (MMIfaceModem *iface);
 static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
 static void shared_telit_init (MMSharedTelit *iface);
+static void iface_modem_location_init (MMIfaceModemLocation *iface);
 
 static MMIfaceModem *iface_modem_parent;
 static MMIfaceModem3gpp *iface_modem_3gpp_parent;
+static MMIfaceModemLocation *iface_modem_location_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemTelit, mm_broadband_modem_telit, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_SHARED_TELIT, shared_telit_init));
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_SHARED_TELIT, shared_telit_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_LOCATION, iface_modem_location_init));
 
 #define CSIM_UNLOCK_MAX_TIMEOUT 3
 
@@ -62,8 +66,343 @@ struct _MMBroadbandModemTelitPrivate {
     GTask *csim_lock_task;
     guint csim_lock_timeout_id;
     gboolean parse_qss;
+    MMModemLocationSource enabled_sources;
 };
 
+
+typedef struct {
+    MMModemLocationSource source;
+    guint gps_enable_step;
+} LocationGatheringContext;
+
+/*
+ * AT$GPSNMUN
+ *    enable:  0 NMEA stream disabled (default)
+ *             1 NMEA stream enabled in the form $GPSNMUN: <nmea sentence><CR>
+ *             2 NMEA stream enabled in the form <nmea sentence><CR>
+ *             3 dedicated NMEA stream
+ *    GGA:     0 disable (default), 1 enable
+ *    GLL:     0 disable (default), 1 enable
+ *    GSA:     0 disable (default), 1 enable
+ *    GSV:     0 disable (default), 1 enable
+ *    RMC:     0 disable (default), 1 enable
+ *    VTG:     0 disable (default), 1 enable
+ */
+static const gchar *gps_enable[] = {
+    "$GPSP=1",
+    "$GPSNMUN=2,1,1,1,1,1,1"
+};
+
+static gboolean
+disable_location_gathering_finish (MMIfaceModemLocation *self,
+                                   GAsyncResult *res,
+                                   GError **error)
+{
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
+
+static void
+gps_disabled_ready (MMBaseModem *self,
+                    GAsyncResult *res,
+                    GTask *task)
+{
+    LocationGatheringContext *ctx;
+    MMPortSerialGps *gps_port;
+    GError *error = NULL;
+
+    mm_base_modem_at_command_finish (self, res, &error);
+    ctx = g_task_get_task_data (task);
+    /* Only use the GPS port in NMEA/RAW setups */
+    if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                       MM_MODEM_LOCATION_SOURCE_GPS_RAW)) {
+        /* Even if we get an error here, we try to close the GPS port */
+        gps_port = mm_base_modem_peek_port_gps (self);
+        if (gps_port)
+            mm_port_serial_close (MM_PORT_SERIAL (gps_port));
+    }
+
+    if (error)
+        g_task_return_error (task, error);
+    else
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
+}
+
+static void
+disable_location_gathering (MMIfaceModemLocation *self,
+                            MMModemLocationSource source,
+                            GAsyncReadyCallback callback,
+                            gpointer user_data)
+{
+    MMBroadbandModemTelit *telit = MM_BROADBAND_MODEM_TELIT (self);
+    gboolean stop_gps = FALSE;
+    LocationGatheringContext *ctx;
+    GTask *task;
+
+    ctx = g_new (LocationGatheringContext, 1);
+    ctx->source = source;
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
+    /* Only stop GPS engine if no GPS-related sources enabled */
+    if (source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                  MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                  MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)) {
+        telit->priv->enabled_sources &= ~source;
+
+        if (!(telit->priv->enabled_sources & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                                            MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                                            MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)))
+            stop_gps = TRUE;
+    }
+
+    if (stop_gps) {
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                  "$GPSP=0",
+                                  3,
+                                  FALSE,
+                                  (GAsyncReadyCallback)gps_disabled_ready,
+                                  task);
+        return;
+    }
+    /* For any other location (e.g. 3GPP), or if still some GPS needed, just return */
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
+}
+
+static void
+gps_enabled_ready (MMBaseModem *self,
+                   GAsyncResult *res,
+                   GTask *task)
+{
+    LocationGatheringContext *ctx;
+    GError *error = NULL;
+
+    ctx = g_task_get_task_data (task);
+    if (!mm_base_modem_at_command_finish (self, res, &error)) {
+        g_prefix_error (&error, "couldn't power up GNSS controller: ");
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+    /* After Receiver was powered up we still have to enable unsolicited NMEA events */
+    if (ctx->gps_enable_step < G_N_ELEMENTS (gps_enable)) {
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                  gps_enable[ctx->gps_enable_step++],
+                                  3,
+                                  FALSE,
+                                  (GAsyncReadyCallback)gps_enabled_ready,
+                                  task);
+        return;
+    }
+
+    mm_obj_dbg (self, "GNSS controller is powered up");
+
+    /* Only use the GPS port in NMEA/RAW setups */
+    if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                       MM_MODEM_LOCATION_SOURCE_GPS_RAW)) {
+        MMPortSerialGps *gps_port;
+
+        gps_port = mm_base_modem_peek_port_gps (self);
+        if (!gps_port ||
+            !mm_port_serial_open (MM_PORT_SERIAL (gps_port), &error)) {
+            if (error)
+                g_task_return_error (task, error);
+            else
+                g_task_return_new_error (task,
+                                         MM_CORE_ERROR,
+                                         MM_CORE_ERROR_FAILED,
+                                         "Couldn't open raw GPS serial port");
+        } else
+            g_task_return_boolean (task, TRUE);
+    } else
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
+}
+
+static void
+parent_enable_location_gathering_ready (MMIfaceModemLocation *_self,
+                                        GAsyncResult *res,
+                                        GTask *task)
+{
+    MMBroadbandModemTelit *self = MM_BROADBAND_MODEM_TELIT (_self);
+    LocationGatheringContext *ctx;
+    gboolean start_gps = FALSE;
+    GError *error = NULL;
+
+    if (!iface_modem_location_parent->enable_location_gathering_finish (_self, res, &error)) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+    /* Now our own enabling */
+    ctx = g_task_get_task_data (task);
+
+    /* NMEA, RAW and UNMANAGED are all enabled in the same way */
+    if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                       MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                       MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)) {
+        /* Only start GPS engine if not done already */
+        if (!(self->priv->enabled_sources & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                                             MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                                             MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)))
+            start_gps = TRUE;
+        self->priv->enabled_sources |= ctx->source;
+    }
+
+    if (start_gps && ctx->gps_enable_step < G_N_ELEMENTS (gps_enable)) {
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                  gps_enable[ctx->gps_enable_step++],
+                                  3,
+                                  FALSE,
+                                  (GAsyncReadyCallback)gps_enabled_ready,
+                                  task);
+        return;
+    }
+    /* For any other location (e.g. 3GPP), or if GPS already running just return */
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
+}
+
+static void
+enable_location_gathering (MMIfaceModemLocation *self,
+                           MMModemLocationSource source,
+                           GAsyncReadyCallback callback,
+                           gpointer user_data)
+{
+    LocationGatheringContext *ctx;
+    GTask *task;
+
+    ctx = g_new (LocationGatheringContext, 1);
+    ctx->source = source;
+    ctx->gps_enable_step = 0;
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
+    /* Chain up parent's gathering enable */
+    iface_modem_location_parent->enable_location_gathering (
+        self,
+        source,
+        (GAsyncReadyCallback)parent_enable_location_gathering_ready,
+        task);
+}
+
+static void
+trace_received (MMPortSerialGps *port,
+                const gchar *trace,
+                MMIfaceModemLocation *self)
+{
+    mm_iface_modem_location_gps_update (self, trace);
+}
+
+static gboolean
+enable_location_gathering_finish (MMIfaceModemLocation *self,
+                                  GAsyncResult *res,
+                                  GError **error)
+{
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
+
+static void
+setup_ports (MMBroadbandModem *self)
+{
+    MMPortSerialGps *gps_data_port;
+
+    /* Call parent's setup ports first always */
+    MM_BROADBAND_MODEM_CLASS (mm_broadband_modem_telit_parent_class)->setup_ports (self);
+
+    gps_data_port = mm_base_modem_peek_port_gps (MM_BASE_MODEM (self));
+    if (gps_data_port) {
+        /* It may happen that the modem was started with GPS already enabled,
+         * in this case GPSP AT command returns always error. Disable it for consistency
+         */
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                  "$GPSP=0", 3, FALSE, FALSE, NULL);
+
+        /* Add handler for the NMEA traces */
+        mm_port_serial_gps_add_trace_handler (gps_data_port,
+                                              (MMPortSerialGpsTraceFn)trace_received,
+                                              self,
+                                              NULL);
+    }
+}
+
+static MMModemLocationSource
+location_load_capabilities_finish (MMIfaceModemLocation *self,
+                                   GAsyncResult *res,
+                                   GError **error)
+{
+    GError *inner_error = NULL;
+    gssize value;
+
+    value = g_task_propagate_int (G_TASK (res), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return MM_MODEM_LOCATION_SOURCE_NONE;
+    }
+    return (MMModemLocationSource)value;
+}
+
+static void
+gpsp_test_ready (MMIfaceModemLocation *self,
+                 GAsyncResult *res,
+                 GTask *task)
+{
+    GError *error = NULL;
+    MMModemLocationSource sources;
+
+    sources = GPOINTER_TO_UINT (g_task_get_task_data (task));
+    mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
+    if (error) {
+        mm_obj_dbg (self, "GPS controller not supported: %s", error->message);
+        g_clear_error (&error);
+    } else if (mm_base_modem_get_port_gps (MM_BASE_MODEM (self)))
+        sources |= (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                    MM_MODEM_LOCATION_SOURCE_GPS_RAW  |
+                    MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED);
+
+    g_task_return_int (task, sources);
+    g_object_unref (task);
+}
+
+static void
+parent_load_capabilities_ready (MMIfaceModemLocation *self,
+                                GAsyncResult *res,
+                                GTask *task)
+{
+    MMModemLocationSource sources;
+    GError *error = NULL;
+
+    sources = iface_modem_location_parent->load_capabilities_finish (self, res, &error);
+    if (error) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+    g_task_set_task_data (task, GUINT_TO_POINTER (sources), NULL);
+    mm_base_modem_at_command (MM_BASE_MODEM (self),
+                              "$GPSP=?",
+                              3,
+                              TRUE,
+                              (GAsyncReadyCallback)gpsp_test_ready,
+                              task);
+}
+
+static void
+location_load_capabilities (MMIfaceModemLocation *self,
+                            GAsyncReadyCallback callback,
+                            gpointer user_data)
+{
+    /* Chain up parent's setup */
+    iface_modem_location_parent->load_capabilities (
+        self,
+        (GAsyncReadyCallback)parent_load_capabilities_ready,
+        g_task_new (self, NULL, callback, user_data));
+}
+
 /*****************************************************************************/
 /* After Sim Unlock (Modem interface) */
 
@@ -136,12 +475,12 @@ telit_qss_unsolicited_handler (MMPortSerialAt *port,
     if (self->priv->csim_lock_state >= CSIM_LOCK_STATE_LOCK_REQUESTED) {
 
         if (prev_qss_status > QSS_STATUS_SIM_REMOVED && cur_qss_status == QSS_STATUS_SIM_REMOVED) {
-            mm_dbg ("QSS handler: #QSS=0 after +CSIM=1 -> CSIM locked!");
+            mm_obj_dbg (self, "QSS handler: #QSS=0 after +CSIM=1: CSIM locked!");
             self->priv->csim_lock_state = CSIM_LOCK_STATE_LOCKED;
         }
 
         if (prev_qss_status == QSS_STATUS_SIM_REMOVED && cur_qss_status != QSS_STATUS_SIM_REMOVED) {
-            mm_dbg ("QSS handler: #QSS>=1 after +CSIM=0 -> CSIM unlocked!");
+            mm_obj_dbg (self, "QSS handler: #QSS>=1 after +CSIM=0: CSIM unlocked!");
             self->priv->csim_lock_state = CSIM_LOCK_STATE_UNLOCKED;
 
             if (self->priv->csim_lock_timeout_id) {
@@ -156,18 +495,18 @@ telit_qss_unsolicited_handler (MMPortSerialAt *port,
     }
 
     if (cur_qss_status != prev_qss_status)
-        mm_dbg ("QSS handler: status changed '%s -> %s'",
-                mm_telit_qss_status_get_string (prev_qss_status),
-                mm_telit_qss_status_get_string (cur_qss_status));
+        mm_obj_dbg (self, "QSS handler: status changed %s -> %s",
+                    mm_telit_qss_status_get_string (prev_qss_status),
+                    mm_telit_qss_status_get_string (cur_qss_status));
 
     if (self->priv->parse_qss == FALSE) {
-        mm_dbg ("QSS: message ignored");
+        mm_obj_dbg (self, "QSS handler: message ignored");
         return;
     }
 
     if ((prev_qss_status == QSS_STATUS_SIM_REMOVED && cur_qss_status != QSS_STATUS_SIM_REMOVED) ||
         (prev_qss_status > QSS_STATUS_SIM_REMOVED && cur_qss_status == QSS_STATUS_SIM_REMOVED)) {
-        mm_info ("QSS handler: SIM swap detected");
+        mm_obj_info (self, "QSS handler: SIM swap detected");
         mm_broadband_modem_update_sim_hot_swap_detected (MM_BROADBAND_MODEM (self));
     }
 }
@@ -212,7 +551,7 @@ telit_qss_enable_ready (MMBaseModem *self,
         g_assert_not_reached ();
 
     if (!mm_base_modem_at_command_full_finish (self, res, error)) {
-        mm_warn ("QSS: error enabling unsolicited on port %s: %s", mm_port_get_device (MM_PORT (port)), (*error)->message);
+        mm_obj_warn (self, "QSS: error enabling unsolicited on port %s: %s", mm_port_get_device (MM_PORT (port)), (*error)->message);
         goto next_step;
     }
 
@@ -247,19 +586,19 @@ telit_qss_query_ready (MMBaseModem *_self,
 
     response = mm_base_modem_at_command_finish (_self, res, &error);
     if (error) {
-        mm_warn ("Could not get \"#QSS?\" reply: %s", error->message);
+        mm_obj_warn (self, "could not get \"#QSS?\" reply: %s", error->message);
         g_error_free (error);
         goto next_step;
     }
 
     qss_status = mm_telit_parse_qss_query (response, &error);
     if (error) {
-        mm_warn ("QSS query parse error: %s", error->message);
+        mm_obj_warn (self, "QSS query parse error: %s", error->message);
         g_error_free (error);
         goto next_step;
     }
 
-    mm_info ("QSS: current status is '%s'", mm_telit_qss_status_get_string (qss_status));
+    mm_obj_dbg (self, "QSS: current status is '%s'", mm_telit_qss_status_get_string (qss_status));
     self->priv->qss_status = qss_status;
 
 next_step:
@@ -278,8 +617,8 @@ qss_setup_step (GTask *task)
 
     switch (ctx->step) {
         case QSS_SETUP_STEP_FIRST:
-            /* Fall back on next step */
             ctx->step++;
+            /* fall through */
         case QSS_SETUP_STEP_QUERY:
             mm_base_modem_at_command (MM_BASE_MODEM (self),
                                       "#QSS?",
@@ -312,8 +651,8 @@ qss_setup_step (GTask *task)
                                                task);
                 return;
             }
-            /* Fall back to next step */
             ctx->step++;
+            /* fall through */
         case QSS_SETUP_STEP_LAST:
             /* If all enabling actions failed (either both, or only primary if
              * there is no secondary), then we return an error */
@@ -327,6 +666,9 @@ qss_setup_step (GTask *task)
                 g_task_return_boolean (task, TRUE);
             g_object_unref (task);
             break;
+
+        default:
+            g_assert_not_reached ();
     }
 }
 
@@ -414,7 +756,7 @@ csim_unlock_ready (MMBaseModem  *_self,
                              MM_MOBILE_EQUIPMENT_ERROR_NOT_SUPPORTED)) {
             self->priv->csim_lock_support = FEATURE_NOT_SUPPORTED;
         }
-        mm_warn ("Couldn't unlock SIM card: %s", error->message);
+        mm_obj_warn (self, "couldn't unlock SIM card: %s", error->message);
         g_error_free (error);
     }
 
@@ -436,7 +778,7 @@ parent_load_unlock_retries_ready (MMIfaceModem *self,
     ctx = g_task_get_task_data (task);
 
     if (!(ctx->retries = iface_modem_parent->load_unlock_retries_finish (self, res, &error))) {
-        mm_warn ("couldn't load unlock retries with generic logic: %s", error->message);
+        mm_obj_warn (self, "couldn't load unlock retries with generic logic: %s", error->message);
         g_error_free (error);
     }
 
@@ -463,7 +805,7 @@ csim_lock_ready (MMBaseModem  *_self,
                              MM_MOBILE_EQUIPMENT_ERROR,
                              MM_MOBILE_EQUIPMENT_ERROR_NOT_SUPPORTED)) {
             self->priv->csim_lock_support = FEATURE_NOT_SUPPORTED;
-            mm_warn ("Couldn't lock SIM card: %s. Continuing without CSIM lock.", error->message);
+            mm_obj_warn (self, "couldn't lock SIM card: %s; continuing without CSIM lock", error->message);
             g_error_free (error);
         } else {
             g_prefix_error (&error, "Couldn't lock SIM card: ");
@@ -513,8 +855,8 @@ handle_csim_locking (GTask    *task,
             }
             break;
         case FEATURE_NOT_SUPPORTED:
-            mm_dbg ("CSIM lock not supported by this modem. Skipping %s command",
-                    is_lock ? "lock" : "unlock");
+            mm_obj_dbg (self, "CSIM lock not supported by this modem; skipping %s command",
+                        is_lock ? "lock" : "unlock");
             ctx->step++;
             load_unlock_retries_step (task);
             break;
@@ -544,9 +886,8 @@ pending_csim_unlock_complete (MMBroadbandModemTelit *self)
 static gboolean
 csim_unlock_periodic_check (MMBroadbandModemTelit *self)
 {
-    if (self->priv->csim_lock_state != CSIM_LOCK_STATE_UNLOCKED) {
-        mm_warn ("CSIM is still locked after %d seconds. Trying to continue anyway", CSIM_UNLOCK_MAX_TIMEOUT);
-    }
+    if (self->priv->csim_lock_state != CSIM_LOCK_STATE_UNLOCKED)
+        mm_obj_warn (self, "CSIM is still locked after %d seconds; trying to continue anyway", CSIM_UNLOCK_MAX_TIMEOUT);
 
     self->priv->csim_lock_timeout_id = 0;
     pending_csim_unlock_complete (self);
@@ -565,8 +906,8 @@ load_unlock_retries_step (GTask *task)
     ctx = g_task_get_task_data (task);
     switch (ctx->step) {
         case LOAD_UNLOCK_RETRIES_STEP_FIRST:
-            /* Fall back on next step */
             ctx->step++;
+            /* fall through */
         case LOAD_UNLOCK_RETRIES_STEP_LOCK:
             handle_csim_locking (task, TRUE);
             break;
@@ -582,7 +923,7 @@ load_unlock_retries_step (GTask *task)
         case LOAD_UNLOCK_RETRIES_STEP_LAST:
             self->priv->csim_lock_task = task;
             if (self->priv->csim_lock_state == CSIM_LOCK_STATE_LOCKED) {
-                mm_dbg ("CSIM is locked. Waiting for #QSS=1");
+                mm_obj_dbg (self, "CSIM is locked, waiting for #QSS=1");
                 self->priv->csim_lock_timeout_id = g_timeout_add_seconds (CSIM_UNLOCK_MAX_TIMEOUT,
                                                                           (GSourceFunc) csim_unlock_periodic_check,
                                                                           g_object_ref(self));
@@ -637,7 +978,7 @@ modem_after_power_up (MMIfaceModem *self,
 
     task = g_task_new (self, NULL, callback, user_data);
 
-    mm_dbg ("Stop ignoring #QSS");
+    mm_obj_dbg (self, "stop ignoring #QSS");
     modem->priv->parse_qss = TRUE;
 
     g_task_return_boolean (task, TRUE);
@@ -663,12 +1004,12 @@ telit_modem_power_down_ready (MMBaseModem *self,
     GError *error = NULL;
 
     if (mm_base_modem_at_command_finish (self, res, &error)) {
-        mm_dbg ("Ignore #QSS unsolicited during power down/low");
+        mm_obj_dbg (self, "sgnore #QSS unsolicited during power down/low");
         MM_BROADBAND_MODEM_TELIT (self)->priv->parse_qss = FALSE;
     }
 
     if (error) {
-        mm_err ("modem power down: %s", error->message);
+        mm_obj_warn (self, "failed modem power down: %s", error->message);
         g_clear_error (&error);
     }
 
@@ -844,8 +1185,8 @@ response_processor_service_ignore_at_errors (MMBaseModem *self,
 }
 
 static const MMBaseModemAtCommand access_tech_commands[] = {
-    { "#PSNT?",  3, TRUE, response_processor_psnt_ignore_at_errors },
-    { "+SERVICE?", 3, TRUE, response_processor_service_ignore_at_errors },
+    { "#PSNT?",    3, FALSE, response_processor_psnt_ignore_at_errors },
+    { "+SERVICE?", 3, FALSE, response_processor_service_ignore_at_errors },
     { NULL }
 };
 
@@ -854,7 +1195,6 @@ load_access_technologies (MMIfaceModem *self,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
-    mm_dbg ("loading access technology (Telit)...");
     mm_base_modem_at_sequence (
         MM_BASE_MODEM (self),
         access_tech_commands,
@@ -901,7 +1241,7 @@ parent_load_supported_modes_ready (MMIfaceModem *self,
 
     /* Filter out those unsupported modes */
     combinations = mm_telit_build_modes_list();
-    filtered = mm_filter_supported_modes (all, combinations);
+    filtered = mm_filter_supported_modes (all, combinations, self);
     g_array_unref (all);
     g_array_unref (combinations);
 
@@ -940,7 +1280,7 @@ cind_set_ready (MMBaseModem  *self,
     GError *error = NULL;
 
     if (!mm_base_modem_at_command_finish (self, res, &error)) {
-        mm_warn ("Couldn't enable custom +CIND settings: %s", error->message);
+        mm_obj_warn (self, "couldn't enable custom +CIND settings: %s", error->message);
         g_error_free (error);
     }
 
@@ -956,7 +1296,7 @@ parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->enable_unsolicited_events_finish (self, res, &error)) {
-        mm_warn ("Couldn't enable parent 3GPP unsolicited events: %s", error->message);
+        mm_obj_warn (self, "couldn't enable parent 3GPP unsolicited events: %s", error->message);
         g_error_free (error);
     }
 
@@ -1031,9 +1371,9 @@ iface_modem_init (MMIfaceModem *iface)
     iface->set_current_bands = mm_shared_telit_modem_set_current_bands;
     iface->set_current_bands_finish = mm_shared_telit_modem_set_current_bands_finish;
     iface->load_current_bands = mm_shared_telit_modem_load_current_bands;
-    iface->load_current_bands_finish = mm_shared_telit_modem_load_bands_finish;
+    iface->load_current_bands_finish = mm_shared_telit_modem_load_current_bands_finish;
     iface->load_supported_bands = mm_shared_telit_modem_load_supported_bands;
-    iface->load_supported_bands_finish = mm_shared_telit_modem_load_bands_finish;
+    iface->load_supported_bands_finish = mm_shared_telit_modem_load_supported_bands_finish;
     iface->load_unlock_retries_finish = modem_load_unlock_retries_finish;
     iface->load_unlock_retries = modem_load_unlock_retries;
     iface->reset = modem_reset;
@@ -1070,10 +1410,25 @@ shared_telit_init (MMSharedTelit *iface)
 {
 }
 
+static void
+iface_modem_location_init (MMIfaceModemLocation *iface)
+{
+    iface_modem_location_parent = g_type_interface_peek_parent (iface);
+
+    iface->load_capabilities = location_load_capabilities;
+    iface->load_capabilities_finish = location_load_capabilities_finish;
+    iface->enable_location_gathering = enable_location_gathering;
+    iface->enable_location_gathering_finish = enable_location_gathering_finish;
+    iface->disable_location_gathering = disable_location_gathering;
+    iface->disable_location_gathering_finish = disable_location_gathering_finish;
+}
+
 static void
 mm_broadband_modem_telit_class_init (MMBroadbandModemTelitClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    MMBroadbandModemClass *broadband_modem_class = MM_BROADBAND_MODEM_CLASS (klass);
 
     g_type_class_add_private (object_class, sizeof (MMBroadbandModemTelitPrivate));
+    broadband_modem_class->setup_ports = setup_ports;
 }
