@@ -21,9 +21,11 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
+#include <ModemManager-tags.h>
+
 #include "mm-kernel-device-generic.h"
 #include "mm-kernel-device-generic-rules.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 
 #if !defined UDEVRULESDIR
 # error UDEVRULESDIR is not defined
@@ -57,9 +59,11 @@ struct _MMKernelDeviceGenericPrivate {
     guint8   interface_subclass;
     guint8   interface_protocol;
     guint8   interface_number;
+    gchar   *interface_description;
     gchar   *physdev_sysfs_path;
     guint16  physdev_vid;
     guint16  physdev_pid;
+    guint16  physdev_revision;
     gchar   *physdev_subsystem;
     gchar   *physdev_manufacturer;
     gchar   *physdev_product;
@@ -120,21 +124,18 @@ preload_sysfs_path (MMKernelDeviceGeneric *self)
                            mm_kernel_event_properties_get_subsystem (self->priv->properties),
                            mm_kernel_event_properties_get_name      (self->priv->properties));
 
-    self->priv->sysfs_path = canonicalize_file_name (tmp);
+    self->priv->sysfs_path = realpath (tmp, NULL);
     if (!self->priv->sysfs_path || !g_file_test (self->priv->sysfs_path, G_FILE_TEST_EXISTS)) {
-        mm_warn ("Invalid sysfs path read for %s/%s",
-                 mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                 mm_kernel_event_properties_get_name      (self->priv->properties));
+        mm_obj_warn (self, "invalid sysfs path read for %s/%s",
+                     mm_kernel_event_properties_get_subsystem (self->priv->properties),
+                     mm_kernel_event_properties_get_name      (self->priv->properties));
         g_clear_pointer (&self->priv->sysfs_path, g_free);
     }
 
     if (self->priv->sysfs_path) {
         const gchar *devpath;
 
-        mm_dbg ("(%s/%s) sysfs path: %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->sysfs_path);
+        mm_obj_dbg (self, "sysfs path: %s", self->priv->sysfs_path);
         devpath = (g_str_has_prefix (self->priv->sysfs_path, "/sys") ?
                    &self->priv->sysfs_path[4] :
                    self->priv->sysfs_path);
@@ -163,7 +164,7 @@ preload_interface_sysfs_path (MMKernelDeviceGeneric *self)
      * The correct parent dir we want to have is the first one with "usb" subsystem.
      */
     aux = g_strdup_printf ("%s/device", self->priv->sysfs_path);
-    dirpath = canonicalize_file_name (aux);
+    dirpath = realpath (aux, NULL);
     g_free (aux);
 
     while (dirpath) {
@@ -179,7 +180,7 @@ preload_interface_sysfs_path (MMKernelDeviceGeneric *self)
             gchar *canonicalized_subsystem;
             gchar *subsystem_name;
 
-            canonicalized_subsystem = canonicalize_file_name (subsystem_filepath);
+            canonicalized_subsystem = realpath (subsystem_filepath, NULL);
             g_free (subsystem_filepath);
 
             subsystem_name = g_path_get_basename (canonicalized_subsystem);
@@ -206,10 +207,7 @@ preload_interface_sysfs_path (MMKernelDeviceGeneric *self)
     }
 
     if (self->priv->interface_sysfs_path)
-        mm_dbg ("(%s/%s) interface sysfs path: %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->interface_sysfs_path);
+        mm_obj_dbg (self, "interface sysfs path: %s", self->priv->interface_sysfs_path);
 }
 
 static void
@@ -225,10 +223,7 @@ preload_physdev_sysfs_path (MMKernelDeviceGeneric *self)
         self->priv->physdev_sysfs_path = g_path_get_dirname (self->priv->interface_sysfs_path);
 
     if (self->priv->physdev_sysfs_path)
-        mm_dbg ("(%s/%s) physdev sysfs path: %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->physdev_sysfs_path);
+        mm_obj_dbg (self, "physdev sysfs path: %s", self->priv->physdev_sysfs_path);
 }
 
 static void
@@ -239,7 +234,7 @@ preload_driver (MMKernelDeviceGeneric *self)
         gchar *tmp2;
 
         tmp = g_strdup_printf ("%s/driver", self->priv->interface_sysfs_path);
-        tmp2 = canonicalize_file_name (tmp);
+        tmp2 = realpath (tmp, NULL);
         if (tmp2 && g_file_test (tmp2, G_FILE_TEST_EXISTS))
             self->priv->driver = g_path_get_basename (tmp2);
         g_free (tmp2);
@@ -247,10 +242,7 @@ preload_driver (MMKernelDeviceGeneric *self)
     }
 
     if (self->priv->driver)
-        mm_dbg ("(%s/%s) driver: %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->driver);
+        mm_obj_dbg (self, "driver: %s", self->priv->driver);
 }
 
 static void
@@ -265,15 +257,10 @@ preload_physdev_vid (MMKernelDeviceGeneric *self)
     }
 
     if (self->priv->physdev_vid) {
-        mm_dbg ("(%s/%s) vid (ID_VENDOR_ID): 0x%04x",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->physdev_vid);
+        mm_obj_dbg (self, "vid (ID_VENDOR_ID): 0x%04x", self->priv->physdev_vid);
         g_object_set_data_full (G_OBJECT (self), "ID_VENDOR_ID", g_strdup_printf ("%04x", self->priv->physdev_vid), g_free);
     } else
-        mm_dbg ("(%s/%s) vid: unknown",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties));
+        mm_obj_dbg (self, "vid: unknown");
 
 }
 
@@ -289,15 +276,28 @@ preload_physdev_pid (MMKernelDeviceGeneric *self)
     }
 
     if (self->priv->physdev_pid) {
-        mm_dbg ("(%s/%s) pid (ID_MODEL_ID): 0x%04x",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->physdev_pid);
+        mm_obj_dbg (self, "pid (ID_MODEL_ID): 0x%04x", self->priv->physdev_pid);
         g_object_set_data_full (G_OBJECT (self), "ID_MODEL_ID", g_strdup_printf ("%04x", self->priv->physdev_pid), g_free);
     } else
-        mm_dbg ("(%s/%s) pid: unknown",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties));
+        mm_obj_dbg (self, "pid: unknown");
+}
+
+static void
+preload_physdev_revision (MMKernelDeviceGeneric *self)
+{
+    if (!self->priv->physdev_revision && self->priv->physdev_sysfs_path) {
+        guint val;
+
+        val = read_sysfs_property_as_hex (self->priv->physdev_sysfs_path, "bcdDevice");
+        if (val && val <= G_MAXUINT16)
+            self->priv->physdev_revision = val;
+    }
+
+    if (self->priv->physdev_revision) {
+        mm_obj_dbg (self, "revision (ID_REVISION): 0x%04x", self->priv->physdev_revision);
+        g_object_set_data_full (G_OBJECT (self), "ID_REVISION", g_strdup_printf ("%04x", self->priv->physdev_revision), g_free);
+    } else
+        mm_obj_dbg (self, "revision: unknown");
 }
 
 static void
@@ -308,16 +308,13 @@ preload_physdev_subsystem (MMKernelDeviceGeneric *self)
         gchar *subsyspath;
 
         aux = g_strdup_printf ("%s/subsystem", self->priv->physdev_sysfs_path);
-        subsyspath = canonicalize_file_name (aux);
+        subsyspath = realpath (aux, NULL);
         self->priv->physdev_subsystem = g_path_get_dirname (subsyspath);
         g_free (subsyspath);
         g_free (aux);
     }
 
-    mm_dbg ("(%s/%s) subsystem: %s",
-            mm_kernel_event_properties_get_subsystem (self->priv->properties),
-            mm_kernel_event_properties_get_name      (self->priv->properties),
-            self->priv->physdev_subsystem ? self->priv->physdev_subsystem : "unknown");
+    mm_obj_dbg (self, "subsystem: %s", self->priv->physdev_subsystem ? self->priv->physdev_subsystem : "unknown");
 }
 
 static void
@@ -327,15 +324,10 @@ preload_manufacturer (MMKernelDeviceGeneric *self)
         self->priv->physdev_manufacturer = (self->priv->physdev_sysfs_path ? read_sysfs_property_as_string (self->priv->physdev_sysfs_path, "manufacturer") : NULL);
 
     if (self->priv->physdev_manufacturer) {
-        mm_dbg ("(%s/%s) manufacturer (ID_VENDOR): %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->physdev_manufacturer);
+        mm_obj_dbg (self, "manufacturer (ID_VENDOR): %s", self->priv->physdev_manufacturer);
         g_object_set_data_full (G_OBJECT (self), "ID_VENDOR", g_strdup (self->priv->physdev_manufacturer), g_free);
     } else
-        mm_dbg ("(%s/%s) manufacturer: unknown",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties));
+        mm_obj_dbg (self, "manufacturer: unknown");
 }
 
 static void
@@ -345,15 +337,10 @@ preload_product (MMKernelDeviceGeneric *self)
         self->priv->physdev_product = (self->priv->physdev_sysfs_path ? read_sysfs_property_as_string (self->priv->physdev_sysfs_path, "product") : NULL);
 
     if (self->priv->physdev_product) {
-        mm_dbg ("(%s/%s) product (ID_MODEL): %s",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->physdev_product);
+        mm_obj_dbg (self, "product (ID_MODEL): %s", self->priv->physdev_product);
         g_object_set_data_full (G_OBJECT (self), "ID_MODEL", g_strdup (self->priv->physdev_product), g_free);
     } else
-        mm_dbg ("(%s/%s) product: unknown",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties));
+        mm_obj_dbg (self, "product: unknown");
 
 }
 
@@ -361,59 +348,56 @@ static void
 preload_interface_class (MMKernelDeviceGeneric *self)
 {
     self->priv->interface_class = (self->priv->interface_sysfs_path ? read_sysfs_property_as_hex (self->priv->interface_sysfs_path, "bInterfaceClass") : 0x00);
-    mm_dbg ("(%s/%s) interface class: 0x%02x",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->interface_class);
+    mm_obj_dbg (self, "interface class: 0x%02x", self->priv->interface_class);
 }
 
 static void
 preload_interface_subclass (MMKernelDeviceGeneric *self)
 {
     self->priv->interface_subclass = (self->priv->interface_sysfs_path ? read_sysfs_property_as_hex (self->priv->interface_sysfs_path, "bInterfaceSubClass") : 0x00);
-    mm_dbg ("(%s/%s) interface subclass: 0x%02x",
-                mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                mm_kernel_event_properties_get_name      (self->priv->properties),
-                self->priv->interface_subclass);
+    mm_obj_dbg (self, "interface subclass: 0x%02x", self->priv->interface_subclass);
 }
 
 static void
 preload_interface_protocol (MMKernelDeviceGeneric *self)
 {
     self->priv->interface_protocol = (self->priv->interface_sysfs_path ? read_sysfs_property_as_hex (self->priv->interface_sysfs_path, "bInterfaceProtocol") : 0x00);
-    mm_dbg ("(%s/%s) interface protocol: 0x%02x",
-            mm_kernel_event_properties_get_subsystem (self->priv->properties),
-            mm_kernel_event_properties_get_name      (self->priv->properties),
-            self->priv->interface_protocol);
+    mm_obj_dbg (self, "interface protocol: 0x%02x", self->priv->interface_protocol);
 }
 
 static void
 preload_interface_number (MMKernelDeviceGeneric *self)
 {
     self->priv->interface_number = (self->priv->interface_sysfs_path ? read_sysfs_property_as_hex (self->priv->interface_sysfs_path, "bInterfaceNumber") : 0x00);
-    mm_dbg ("(%s/%s) interface number (ID_USB_INTERFACE_NUM): 0x%02x",
-            mm_kernel_event_properties_get_subsystem (self->priv->properties),
-            mm_kernel_event_properties_get_name      (self->priv->properties),
-            self->priv->interface_number);
+    mm_obj_dbg (self, "interface number (ID_USB_INTERFACE_NUM): 0x%02x", self->priv->interface_number);
     g_object_set_data_full (G_OBJECT (self), "ID_USB_INTERFACE_NUM", g_strdup_printf ("%02x", self->priv->interface_number), g_free);
 }
 
+static void
+preload_interface_description (MMKernelDeviceGeneric *self)
+{
+    self->priv->interface_description = (self->priv->interface_sysfs_path ? read_sysfs_property_as_string (self->priv->interface_sysfs_path, "interface") : NULL);
+    mm_obj_dbg (self, "interface description: %s", self->priv->interface_description ? self->priv->interface_description : "unknown");
+}
+
 static void
 preload_contents (MMKernelDeviceGeneric *self)
 {
-    preload_sysfs_path           (self);
-    preload_interface_sysfs_path (self);
-    preload_interface_class      (self);
-    preload_interface_subclass   (self);
-    preload_interface_protocol   (self);
-    preload_interface_number     (self);
-    preload_physdev_sysfs_path   (self);
-    preload_manufacturer         (self);
-    preload_product              (self);
-    preload_driver               (self);
-    preload_physdev_vid          (self);
-    preload_physdev_pid          (self);
-    preload_physdev_subsystem    (self);
+    preload_sysfs_path            (self);
+    preload_interface_sysfs_path  (self);
+    preload_interface_class       (self);
+    preload_interface_subclass    (self);
+    preload_interface_protocol    (self);
+    preload_interface_number      (self);
+    preload_interface_description (self);
+    preload_physdev_sysfs_path    (self);
+    preload_manufacturer          (self);
+    preload_product               (self);
+    preload_driver                (self);
+    preload_physdev_vid           (self);
+    preload_physdev_pid           (self);
+    preload_physdev_revision      (self);
+    preload_physdev_subsystem     (self);
 }
 
 /*****************************************************************************/
@@ -474,6 +458,14 @@ kernel_device_get_interface_sysfs_path (MMKernelDevice *self)
     return MM_KERNEL_DEVICE_GENERIC (self)->priv->interface_sysfs_path;
 }
 
+static const gchar *
+kernel_device_get_interface_description (MMKernelDevice *self)
+{
+    g_return_val_if_fail (MM_IS_KERNEL_DEVICE_GENERIC (self), NULL);
+
+    return MM_KERNEL_DEVICE_GENERIC (self)->priv->interface_description;
+}
+
 static const gchar *
 kernel_device_get_physdev_uid (MMKernelDevice *self)
 {
@@ -486,7 +478,7 @@ kernel_device_get_physdev_uid (MMKernelDevice *self)
         return uid;
 
     /* Try to load from properties set */
-    if ((uid = mm_kernel_device_get_property (self, "ID_MM_PHYSDEV_UID")) != NULL)
+    if ((uid = mm_kernel_device_get_property (self, ID_MM_PHYSDEV_UID)) != NULL)
         return uid;
 
     /* Use physical device path, if any */
@@ -521,6 +513,14 @@ kernel_device_get_physdev_pid (MMKernelDevice *self)
     return MM_KERNEL_DEVICE_GENERIC (self)->priv->physdev_pid;
 }
 
+static guint16
+kernel_device_get_physdev_revision (MMKernelDevice *self)
+{
+    g_return_val_if_fail (MM_IS_KERNEL_DEVICE_GENERIC (self), 0);
+
+    return MM_KERNEL_DEVICE_GENERIC (self)->priv->physdev_revision;
+}
+
 static const gchar *
 kernel_device_get_physdev_sysfs_path (MMKernelDevice *self)
 {
@@ -545,6 +545,14 @@ kernel_device_get_physdev_manufacturer (MMKernelDevice *self)
     return MM_KERNEL_DEVICE_GENERIC (self)->priv->physdev_manufacturer;
 }
 
+static const gchar *
+kernel_device_get_physdev_product (MMKernelDevice *self)
+{
+    g_return_val_if_fail (MM_IS_KERNEL_DEVICE_GENERIC (self), 0);
+
+    return MM_KERNEL_DEVICE_GENERIC (self)->priv->physdev_product;
+}
+
 static gboolean
 kernel_device_cmp (MMKernelDevice *a,
                    MMKernelDevice *b)
@@ -705,7 +713,7 @@ check_condition (MMKernelDeviceGeneric *self,
             result = (g_str_equal (match->value, "?*") || ((mm_get_uint_from_hex_str (match->value, &val)) &&
                                                            ((self->priv->interface_number == val) == condition_equal)));
         else
-            mm_warn ("Unknown attribute: %s", attribute);
+            mm_obj_warn (self, "unknown attribute: %s", attribute);
 
         g_free (contents);
         g_free (attribute);
@@ -727,7 +735,7 @@ check_condition (MMKernelDeviceGeneric *self,
         return result;
     }
 
-    mm_warn ("Unknown match condition parameter: %s", match->parameter);
+    mm_obj_warn (self, "unknown match condition parameter: %s", match->parameter);
     return FALSE;
 }
 
@@ -770,11 +778,9 @@ check_rule (MMKernelDeviceGeneric *self,
                 property_value_read = g_strdup_printf ("%02x", self->priv->interface_number);
 
             /* add new property */
-            mm_dbg ("(%s/%s) property added: %s=%s",
-                    mm_kernel_event_properties_get_subsystem (self->priv->properties),
-                    mm_kernel_event_properties_get_name      (self->priv->properties),
-                    rule->result.content.property.name,
-                    property_value_read ? property_value_read : rule->result.content.property.value);
+            mm_obj_dbg (self, "property added: %s=%s",
+                        rule->result.content.property.name,
+                        property_value_read ? property_value_read : rule->result.content.property.value);
 
             if (!property_value_read)
                 /* NOTE: we keep a reference to the list of rules ourselves, so it isn't
@@ -801,6 +807,7 @@ check_rule (MMKernelDeviceGeneric *self,
 
         case MM_UDEV_RULE_RESULT_TYPE_GOTO_TAG:
         case MM_UDEV_RULE_RESULT_TYPE_UNKNOWN:
+        default:
             g_assert_not_reached ();
         }
     }
@@ -842,9 +849,7 @@ check_preload (MMKernelDeviceGeneric *self)
     if (g_strcmp0 (mm_kernel_event_properties_get_subsystem (self->priv->properties), "virtual") == 0)
         return;
 
-    mm_dbg ("(%s/%s) preloading contents and properties...",
-            mm_kernel_event_properties_get_subsystem (self->priv->properties),
-            mm_kernel_event_properties_get_name      (self->priv->properties));
+    mm_obj_dbg (self, "preloading contents and properties...");
     preload_contents (self);
     preload_properties (self);
 }
@@ -908,29 +913,29 @@ kernel_device_get_property_as_int_hex (MMKernelDevice *self,
 /*****************************************************************************/
 
 MMKernelDevice *
-mm_kernel_device_generic_new_with_rules (MMKernelEventProperties  *properties,
+mm_kernel_device_generic_new_with_rules (MMKernelEventProperties  *props,
                                          GArray                   *rules,
                                          GError                  **error)
 {
-    g_return_val_if_fail (MM_IS_KERNEL_EVENT_PROPERTIES (properties), NULL);
+    g_return_val_if_fail (MM_IS_KERNEL_EVENT_PROPERTIES (props), NULL);
 
     /* Note: we allow NULL rules, e.g. for virtual devices */
 
     return MM_KERNEL_DEVICE (g_initable_new (MM_TYPE_KERNEL_DEVICE_GENERIC,
                                              NULL,
                                              error,
-                                             "properties", properties,
+                                             "properties", props,
                                              "rules",      rules,
                                              NULL));
 }
 
 MMKernelDevice *
-mm_kernel_device_generic_new (MMKernelEventProperties  *properties,
+mm_kernel_device_generic_new (MMKernelEventProperties  *props,
                               GError                  **error)
 {
     static GArray *rules = NULL;
 
-    g_return_val_if_fail (MM_IS_KERNEL_EVENT_PROPERTIES (properties), NULL);
+    g_return_val_if_fail (MM_IS_KERNEL_EVENT_PROPERTIES (props), NULL);
 
     /* We only try to load the default list of rules once */
     if (G_UNLIKELY (!rules)) {
@@ -939,7 +944,7 @@ mm_kernel_device_generic_new (MMKernelEventProperties  *properties,
             return NULL;
     }
 
-    return mm_kernel_device_generic_new_with_rules (properties, rules, error);
+    return mm_kernel_device_generic_new_with_rules (props, rules, error);
 }
 
 /*****************************************************************************/
@@ -1071,26 +1076,29 @@ mm_kernel_device_generic_class_init (MMKernelDeviceGenericClass *klass)
     object_class->get_property = get_property;
     object_class->set_property = set_property;
 
-    kernel_device_class->get_subsystem            = kernel_device_get_subsystem;
-    kernel_device_class->get_name                 = kernel_device_get_name;
-    kernel_device_class->get_driver               = kernel_device_get_driver;
-    kernel_device_class->get_sysfs_path           = kernel_device_get_sysfs_path;
-    kernel_device_class->get_physdev_uid          = kernel_device_get_physdev_uid;
-    kernel_device_class->get_physdev_vid          = kernel_device_get_physdev_vid;
-    kernel_device_class->get_physdev_pid          = kernel_device_get_physdev_pid;
-    kernel_device_class->get_physdev_sysfs_path   = kernel_device_get_physdev_sysfs_path;
-    kernel_device_class->get_physdev_subsystem    = kernel_device_get_physdev_subsystem;
-    kernel_device_class->get_physdev_manufacturer = kernel_device_get_physdev_manufacturer;
-    kernel_device_class->get_interface_class      = kernel_device_get_interface_class;
-    kernel_device_class->get_interface_subclass   = kernel_device_get_interface_subclass;
-    kernel_device_class->get_interface_protocol   = kernel_device_get_interface_protocol;
-    kernel_device_class->get_interface_sysfs_path = kernel_device_get_interface_sysfs_path;
-    kernel_device_class->cmp                      = kernel_device_cmp;
-    kernel_device_class->has_property             = kernel_device_has_property;
-    kernel_device_class->get_property             = kernel_device_get_property;
-    kernel_device_class->get_property_as_boolean  = kernel_device_get_property_as_boolean;
-    kernel_device_class->get_property_as_int      = kernel_device_get_property_as_int;
-    kernel_device_class->get_property_as_int_hex  = kernel_device_get_property_as_int_hex;
+    kernel_device_class->get_subsystem             = kernel_device_get_subsystem;
+    kernel_device_class->get_name                  = kernel_device_get_name;
+    kernel_device_class->get_driver                = kernel_device_get_driver;
+    kernel_device_class->get_sysfs_path            = kernel_device_get_sysfs_path;
+    kernel_device_class->get_physdev_uid           = kernel_device_get_physdev_uid;
+    kernel_device_class->get_physdev_vid           = kernel_device_get_physdev_vid;
+    kernel_device_class->get_physdev_pid           = kernel_device_get_physdev_pid;
+    kernel_device_class->get_physdev_revision      = kernel_device_get_physdev_revision;
+    kernel_device_class->get_physdev_sysfs_path    = kernel_device_get_physdev_sysfs_path;
+    kernel_device_class->get_physdev_subsystem     = kernel_device_get_physdev_subsystem;
+    kernel_device_class->get_physdev_manufacturer  = kernel_device_get_physdev_manufacturer;
+    kernel_device_class->get_physdev_product       = kernel_device_get_physdev_product;
+    kernel_device_class->get_interface_class       = kernel_device_get_interface_class;
+    kernel_device_class->get_interface_subclass    = kernel_device_get_interface_subclass;
+    kernel_device_class->get_interface_protocol    = kernel_device_get_interface_protocol;
+    kernel_device_class->get_interface_sysfs_path  = kernel_device_get_interface_sysfs_path;
+    kernel_device_class->get_interface_description = kernel_device_get_interface_description;
+    kernel_device_class->cmp                       = kernel_device_cmp;
+    kernel_device_class->has_property              = kernel_device_has_property;
+    kernel_device_class->get_property              = kernel_device_get_property;
+    kernel_device_class->get_property_as_boolean   = kernel_device_get_property_as_boolean;
+    kernel_device_class->get_property_as_int       = kernel_device_get_property_as_int;
+    kernel_device_class->get_property_as_int_hex   = kernel_device_get_property_as_int_hex;
 
     /* Device-wide properties are stored per-port in the generic backend */
     kernel_device_class->has_global_property            = kernel_device_has_property;
