@@ -21,7 +21,8 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
-#include "mm-at-serial-port.h"
+#include "mm-base-bearer.h"
+#include "mm-port-serial-at.h"
 
 #define MM_TYPE_IFACE_MODEM_3GPP               (mm_iface_modem_3gpp_get_type ())
 #define MM_IFACE_MODEM_3GPP(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), MM_TYPE_IFACE_MODEM_3GPP, MMIfaceModem3gpp))
@@ -33,7 +34,9 @@
 #define MM_IFACE_MODEM_3GPP_CS_NETWORK_SUPPORTED    "iface-modem-3gpp-cs-network-supported"
 #define MM_IFACE_MODEM_3GPP_PS_NETWORK_SUPPORTED    "iface-modem-3gpp-ps-network-supported"
 #define MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED   "iface-modem-3gpp-eps-network-supported"
+#define MM_IFACE_MODEM_3GPP_5GS_NETWORK_SUPPORTED   "iface-modem-3gpp-5gs-network-supported"
 #define MM_IFACE_MODEM_3GPP_IGNORED_FACILITY_LOCKS  "iface-modem-3gpp-ignored-facility-locks"
+#define MM_IFACE_MODEM_3GPP_INITIAL_EPS_BEARER      "iface-modem-3gpp-initial-eps-bearer"
 
 #define MM_IFACE_MODEM_3GPP_ALL_ACCESS_TECHNOLOGIES_MASK    \
     (MM_MODEM_ACCESS_TECHNOLOGY_GSM |                       \
@@ -45,7 +48,8 @@
      MM_MODEM_ACCESS_TECHNOLOGY_HSUPA |                     \
      MM_MODEM_ACCESS_TECHNOLOGY_HSPA |                      \
      MM_MODEM_ACCESS_TECHNOLOGY_HSPA_PLUS |                 \
-     MM_MODEM_ACCESS_TECHNOLOGY_LTE)
+     MM_MODEM_ACCESS_TECHNOLOGY_LTE |                       \
+     MM_MODEM_ACCESS_TECHNOLOGY_5GNR)
 
 typedef struct _MMIfaceModem3gpp MMIfaceModem3gpp;
 
@@ -68,6 +72,14 @@ struct _MMIfaceModem3gpp {
                                                                GAsyncResult *res,
                                                                GError **error);
 
+    /* Loading of the UE mode of operation for EPS property */
+    void                          (* load_eps_ue_mode_operation)        (MMIfaceModem3gpp     *self,
+                                                                         GAsyncReadyCallback   callback,
+                                                                         gpointer              user_data);
+    MMModem3gppEpsUeModeOperation (* load_eps_ue_mode_operation_finish) (MMIfaceModem3gpp     *self,
+                                                                         GAsyncResult         *res,
+                                                                         GError              **error);
+
     /* Asynchronous setting up unsolicited events */
     void (*setup_unsolicited_events) (MMIfaceModem3gpp *self,
                                       GAsyncReadyCallback callback,
@@ -137,13 +149,34 @@ struct _MMIfaceModem3gpp {
                                                                 GAsyncResult *res,
                                                                 GError **error);
 
-    /* Run CS/PS/EPS registration state checks..
+    /* Asynchronous initial default EPS bearer loading */
+    void                 (*load_initial_eps_bearer)        (MMIfaceModem3gpp     *self,
+                                                            GAsyncReadyCallback   callback,
+                                                            gpointer              user_data);
+    MMBearerProperties * (*load_initial_eps_bearer_finish) (MMIfaceModem3gpp     *self,
+                                                            GAsyncResult         *res,
+                                                            GError              **error);
+
+    /* Asynchronous initial default EPS bearer settings loading */
+    void                 (*load_initial_eps_bearer_settings)        (MMIfaceModem3gpp     *self,
+                                                                     GAsyncReadyCallback   callback,
+                                                                     gpointer              user_data);
+    MMBearerProperties * (*load_initial_eps_bearer_settings_finish) (MMIfaceModem3gpp     *self,
+                                                                     GAsyncResult         *res,
+                                                                     GError              **error);
+
+    /* Create initial default EPS bearer object */
+    MMBaseBearer * (*create_initial_eps_bearer) (MMIfaceModem3gpp   *self,
+                                                 MMBearerProperties *properties);
+
+    /* Run CS/PS/EPS/5GS registration state checks..
      * Note that no registration state is returned, implementations should call
      * mm_iface_modem_3gpp_update_registration_state(). */
     void (* run_registration_checks) (MMIfaceModem3gpp *self,
-                                      gboolean cs_supported,
-                                      gboolean ps_supported,
-                                      gboolean eps_supported,
+                                      gboolean is_cs_supported,
+                                      gboolean is_ps_supported,
+                                      gboolean is_eps_supported,
+                                      gboolean is_5gs_supported,
                                       GAsyncReadyCallback callback,
                                       gpointer user_data);
     gboolean (*run_registration_checks_finish) (MMIfaceModem3gpp *self,
@@ -176,14 +209,6 @@ struct _MMIfaceModem3gpp {
                                           GAsyncResult *res,
                                           GError **error);
 
-    /* Loading of the subscription state property */
-    void (*load_subscription_state) (MMIfaceModem3gpp *self,
-                                     GAsyncReadyCallback callback,
-                                     gpointer user_data);
-    MMModem3gppSubscriptionState (*load_subscription_state_finish) (MMIfaceModem3gpp *self,
-                                                                    GAsyncResult *res,
-                                                                    GError **error);
-
     /* Scan current networks, expect a GList of MMModem3gppNetworkInfo */
     void (* scan_networks) (MMIfaceModem3gpp *self,
                             GAsyncReadyCallback callback,
@@ -191,6 +216,24 @@ struct _MMIfaceModem3gpp {
     GList * (*scan_networks_finish) (MMIfaceModem3gpp *self,
                                      GAsyncResult *res,
                                      GError **error);
+
+    /* Set UE mode of operation for EPS */
+    void     (* set_eps_ue_mode_operation)        (MMIfaceModem3gpp               *self,
+                                                   MMModem3gppEpsUeModeOperation   mode,
+                                                   GAsyncReadyCallback             callback,
+                                                   gpointer                        user_data);
+    gboolean (* set_eps_ue_mode_operation_finish) (MMIfaceModem3gpp               *self,
+                                                   GAsyncResult                   *res,
+                                                   GError                        **error);
+
+    /* Set initial EPS bearer settings */
+    void     (* set_initial_eps_bearer_settings)        (MMIfaceModem3gpp     *self,
+                                                         MMBearerProperties   *properties,
+                                                         GAsyncReadyCallback   callback,
+                                                         gpointer              user_data);
+    gboolean (* set_initial_eps_bearer_settings_finish) (MMIfaceModem3gpp     *self,
+                                                         GAsyncResult         *res,
+                                                         GError              **error);
 };
 
 GType mm_iface_modem_3gpp_get_type (void);
@@ -234,13 +277,20 @@ void mm_iface_modem_3gpp_update_ps_registration_state (MMIfaceModem3gpp *self,
                                                        MMModem3gppRegistrationState state);
 void mm_iface_modem_3gpp_update_eps_registration_state (MMIfaceModem3gpp *self,
                                                         MMModem3gppRegistrationState state);
+void mm_iface_modem_3gpp_update_5gs_registration_state (MMIfaceModem3gpp *self,
+                                                        MMModem3gppRegistrationState state);
 void mm_iface_modem_3gpp_update_subscription_state (MMIfaceModem3gpp *self,
                                                     MMModem3gppSubscriptionState state);
 void mm_iface_modem_3gpp_update_access_technologies (MMIfaceModem3gpp *self,
                                                      MMModemAccessTechnology access_tech);
 void mm_iface_modem_3gpp_update_location            (MMIfaceModem3gpp *self,
                                                      gulong location_area_code,
+                                                     gulong tracking_area_code,
                                                      gulong cell_id);
+void mm_iface_modem_3gpp_update_pco_list            (MMIfaceModem3gpp *self,
+                                                     const GList *pco_list);
+void mm_iface_modem_3gpp_update_initial_eps_bearer  (MMIfaceModem3gpp *self,
+                                                     MMBearerProperties *properties);
 
 /* Run all registration checks */
 void mm_iface_modem_3gpp_run_registration_checks (MMIfaceModem3gpp *self,
@@ -260,14 +310,23 @@ gboolean mm_iface_modem_3gpp_reload_current_registration_info_finish (MMIfaceMod
 void     mm_iface_modem_3gpp_clear_current_operator                  (MMIfaceModem3gpp *self);
 
 /* Allow registering in the network */
-gboolean mm_iface_modem_3gpp_register_in_network_finish (MMIfaceModem3gpp *self,
-                                                         GAsyncResult *res,
-                                                         GError **error);
-void     mm_iface_modem_3gpp_register_in_network        (MMIfaceModem3gpp *self,
-                                                         const gchar *operator_id,
-                                                         guint max_registration_time,
-                                                         GAsyncReadyCallback callback,
-                                                         gpointer user_data);
+void     mm_iface_modem_3gpp_register_in_network        (MMIfaceModem3gpp    *self,
+                                                         const gchar         *operator_id,
+                                                         gboolean             force_registration,
+                                                         guint                max_registration_time,
+                                                         GAsyncReadyCallback  callback,
+                                                         gpointer             user_data);
+gboolean mm_iface_modem_3gpp_register_in_network_finish (MMIfaceModem3gpp    *self,
+                                                         GAsyncResult        *res,
+                                                         GError             **error);
+
+/* Allow re-registering in the network with last settings */
+void     mm_iface_modem_3gpp_reregister_in_network        (MMIfaceModem3gpp     *self,
+                                                           GAsyncReadyCallback   callback,
+                                                           gpointer              user_data);
+gboolean mm_iface_modem_3gpp_reregister_in_network_finish (MMIfaceModem3gpp     *self,
+                                                           GAsyncResult         *res,
+                                                           GError              **error);
 
 /* Bind properties for simple GetStatus() */
 void mm_iface_modem_3gpp_bind_simple_status (MMIfaceModem3gpp *self,
