@@ -19,7 +19,7 @@
 
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-signal.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 
 #define SUPPORT_CHECKED_TAG "signal-support-checked-tag"
 #define SUPPORTED_TAG       "signal-supported-tag"
@@ -93,7 +93,7 @@ load_values_ready (MMIfaceModemSignal *self,
             &umts,
             &lte,
             &error)) {
-        mm_warn ("Couldn't load extended signal information: %s", error->message);
+        mm_obj_warn (self, "couldn't load extended signal information: %s", error->message);
         g_error_free (error);
         clear_values (self);
         return;
@@ -103,8 +103,7 @@ load_values_ready (MMIfaceModemSignal *self,
                   MM_IFACE_MODEM_SIGNAL_DBUS_SKELETON, &skeleton,
                   NULL);
     if (!skeleton) {
-        mm_warn ("Cannot update extended signal information: "
-                 "Couldn't get interface skeleton");
+        mm_obj_warn (self, "cannot update extended signal information: couldn't get interface skeleton");
         return;
     }
 
@@ -113,35 +112,40 @@ load_values_ready (MMIfaceModemSignal *self,
         mm_gdbus_modem_signal_set_cdma (skeleton, dictionary);
         g_variant_unref (dictionary);
         g_object_unref (cdma);
-    }
+    } else
+        mm_gdbus_modem_signal_set_cdma (skeleton, NULL);
 
     if (evdo) {
         dictionary = mm_signal_get_dictionary (evdo);
         mm_gdbus_modem_signal_set_evdo (skeleton, dictionary);
         g_variant_unref (dictionary);
         g_object_unref (evdo);
-    }
+    } else
+        mm_gdbus_modem_signal_set_evdo (skeleton, NULL);
 
     if (gsm) {
         dictionary = mm_signal_get_dictionary (gsm);
         mm_gdbus_modem_signal_set_gsm (skeleton, dictionary);
         g_variant_unref (dictionary);
         g_object_unref (gsm);
-    }
+    } else
+        mm_gdbus_modem_signal_set_gsm (skeleton, NULL);
 
     if (umts) {
         dictionary = mm_signal_get_dictionary (umts);
         mm_gdbus_modem_signal_set_umts (skeleton, dictionary);
         g_variant_unref (dictionary);
         g_object_unref (umts);
-    }
+    } else
+        mm_gdbus_modem_signal_set_umts (skeleton, NULL);
 
     if (lte) {
         dictionary = mm_signal_get_dictionary (lte);
         mm_gdbus_modem_signal_set_lte (skeleton, dictionary);
         g_variant_unref (dictionary);
         g_object_unref (lte);
-    }
+    } else
+        mm_gdbus_modem_signal_set_lte (skeleton, NULL);
 
     /* Flush right away */
     g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (skeleton));
@@ -163,11 +167,13 @@ refresh_context_cb (MMIfaceModemSignal *self)
 static void
 teardown_refresh_context (MMIfaceModemSignal *self)
 {
-    mm_dbg ("Extended signal information reporting disabled");
     clear_values (self);
     if (G_UNLIKELY (!refresh_context_quark))
         refresh_context_quark  = g_quark_from_static_string (REFRESH_CONTEXT_TAG);
-    g_object_set_qdata (G_OBJECT (self), refresh_context_quark, NULL);
+    if (g_object_get_qdata (G_OBJECT (self), refresh_context_quark)) {
+        mm_obj_dbg (self, "extended signal information reporting disabled");
+        g_object_set_qdata (G_OBJECT (self), refresh_context_quark, NULL);
+    }
 }
 
 static gboolean
@@ -203,14 +209,14 @@ setup_refresh_context (MMIfaceModemSignal *self,
 
     /* User disabling? */
     if (new_rate == 0) {
-        mm_dbg ("Extended signal information reporting disabled (rate: 0 seconds)");
+        mm_obj_dbg (self, "extended signal information reporting disabled (rate: 0 seconds)");
         clear_values (self);
         g_object_set_qdata (G_OBJECT (self), refresh_context_quark, NULL);
         return TRUE;
     }
 
     if (modem_state < MM_MODEM_STATE_ENABLING) {
-        mm_dbg ("Extended signal information reporting disabled (modem not yet enabled)");
+        mm_obj_dbg (self, "extended signal information reporting disabled (modem not yet enabled)");
         return TRUE;
     }
 
@@ -231,7 +237,7 @@ setup_refresh_context (MMIfaceModemSignal *self,
     }
 
     /* Update refresh context */
-    mm_dbg ("Extended signal information reporting enabled (rate: %u seconds)", new_rate);
+    mm_obj_dbg (self, "extended signal information reporting enabled (rate: %u seconds)", new_rate);
     ctx->rate = new_rate;
     if (ctx->timeout_source)
         g_source_remove (ctx->timeout_source);
@@ -395,7 +401,7 @@ check_support_ready (MMIfaceModemSignal *self,
     if (!MM_IFACE_MODEM_SIGNAL_GET_INTERFACE (self)->check_support_finish (self, res, &error)) {
         if (error) {
             /* This error shouldn't be treated as critical */
-            mm_dbg ("Extended signal support check failed: '%s'", error->message);
+            mm_obj_dbg (self, "extended signal support check failed: %s", error->message);
             g_error_free (error);
         }
     } else {
@@ -436,8 +442,8 @@ interface_initialization_step (GTask *task)
             supported_quark = (g_quark_from_static_string (
                                    SUPPORTED_TAG));
 
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_CHECK_SUPPORT:
         if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
@@ -463,8 +469,8 @@ interface_initialization_step (GTask *task)
             /* If there is no implementation to check support, assume we DON'T
              * support it. */
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED:
         if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
@@ -476,8 +482,8 @@ interface_initialization_step (GTask *task)
             g_object_unref (task);
             return;
         }
-        /* Fall down to next step */
         ctx->step++;
+        /* fall through */
 
     case INITIALIZATION_STEP_LAST:
         /* We are done without errors! */
@@ -494,6 +500,9 @@ interface_initialization_step (GTask *task)
         g_task_return_boolean (task, TRUE);
         g_object_unref (task);
         return;
+
+    default:
+        break;
     }
 
     g_assert_not_reached ();
@@ -536,6 +545,9 @@ mm_iface_modem_signal_initialize (MMIfaceModemSignal *self,
 void
 mm_iface_modem_signal_shutdown (MMIfaceModemSignal *self)
 {
+    /* Teardown refresh context */
+    teardown_refresh_context (self);
+
     /* Unexport DBus interface and remove the skeleton */
     mm_gdbus_object_skeleton_set_modem_signal (MM_GDBUS_OBJECT_SKELETON (self), NULL);
     g_object_set (self,
