@@ -34,13 +34,23 @@
 #include "mm-port-serial-qcdm.h"
 #include "mm-serial-parsers.h"
 #include "mm-private-boxed-types.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 #include "mm-daemon-enums-types.h"
 
-G_DEFINE_TYPE (MMPlugin, mm_plugin, G_TYPE_OBJECT)
+#if defined WITH_QMI
+# include "mm-broadband-modem-qmi.h"
+#endif
+#if defined WITH_MBIM
+# include "mm-broadband-modem-mbim.h"
+#endif
+
+static void log_object_iface_init (MMLogObjectInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (MMPlugin, mm_plugin, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
 
 /* Virtual port corresponding to the embedded modem */
-static gchar *virtual_port[] = {"smd0", NULL};
+static const gchar *virtual_port[] = {"smd0", NULL};
 
 #define HAS_POST_PROBING_FILTERS(self)          \
     (self->priv->vendor_strings ||              \
@@ -54,8 +64,9 @@ static gchar *virtual_port[] = {"smd0", NULL};
 
 
 struct _MMPluginPrivate {
-    gchar *name;
+    gchar      *name;
     GHashTable *tasks;
+    gboolean    is_generic;
 
     /* Pre-probing filters */
     gchar **subsystems;
@@ -97,6 +108,7 @@ struct _MMPluginPrivate {
 enum {
     PROP_0,
     PROP_NAME,
+    PROP_IS_GENERIC,
     PROP_ALLOWED_SUBSYSTEMS,
     PROP_ALLOWED_DRIVERS,
     PROP_FORBIDDEN_DRIVERS,
@@ -140,12 +152,24 @@ mm_plugin_get_allowed_udev_tags (MMPlugin *self)
     return (const gchar **) self->priv->udev_tags;
 }
 
+const guint16 *
+mm_plugin_get_allowed_vendor_ids (MMPlugin *self)
+{
+    return self->priv->vendor_ids;
+}
+
 const mm_uint16_pair *
 mm_plugin_get_allowed_product_ids (MMPlugin *self)
 {
     return self->priv->product_ids;
 }
 
+gboolean
+mm_plugin_is_generic (MMPlugin *self)
+{
+    return self->priv->is_generic;
+}
+
 /*****************************************************************************/
 
 static gboolean
@@ -227,9 +251,7 @@ apply_pre_probing_filters (MMPlugin       *self,
     /* The plugin may specify that only some subsystems are supported. If that
      * is the case, filter by subsystem */
     if (apply_subsystem_filter (self, port)) {
-        mm_dbg ("(%s) [%s] filtered by subsystem",
-                self->priv->name,
-                mm_kernel_device_get_name (port));
+        mm_obj_dbg (self, "port %s filtered by subsystem", mm_kernel_device_get_name (port));
         return TRUE;
     }
 
@@ -258,9 +280,7 @@ apply_pre_probing_filters (MMPlugin       *self,
 
         /* If error retrieving driver: unsupported */
         if (!drivers) {
-            mm_dbg ("(%s) [%s] filtered as couldn't retrieve drivers",
-                    self->priv->name,
-                    mm_kernel_device_get_name (port));
+            mm_obj_dbg (self, "port %s filtered as couldn't retrieve drivers", mm_kernel_device_get_name (port));
             return TRUE;
         }
 
@@ -279,9 +299,7 @@ apply_pre_probing_filters (MMPlugin       *self,
 
             /* If we didn't match any driver: unsupported */
             if (!found) {
-                mm_dbg ("(%s) [%s] filtered by drivers",
-                        self->priv->name,
-                        mm_kernel_device_get_name (port));
+                mm_obj_dbg (self, "port %s filtered by drivers", mm_kernel_device_get_name (port));
                 return TRUE;
             }
         }
@@ -294,9 +312,7 @@ apply_pre_probing_filters (MMPlugin       *self,
                 for (j = 0; drivers[j]; j++) {
                     /* If we match a forbidden driver: unsupported */
                     if (g_str_equal (drivers[j], self->priv->forbidden_drivers[i])) {
-                        mm_dbg ("(%s) [%s] filtered by forbidden drivers",
-                                self->priv->name,
-                                mm_kernel_device_get_name (port));
+                        mm_obj_dbg (self, "port %s filtered by forbidden drivers", mm_kernel_device_get_name (port));
                         return TRUE;
                     }
                 }
@@ -310,9 +326,7 @@ apply_pre_probing_filters (MMPlugin       *self,
             for (j = 0; drivers[j]; j++) {
                 /* If we match the QMI driver: unsupported */
                 if (g_str_equal (drivers[j], "qmi_wwan")) {
-                    mm_dbg ("(%s) [%s] filtered by implicit QMI driver",
-                            self->priv->name,
-                            mm_kernel_device_get_name (port));
+                    mm_obj_dbg (self, "port %s filtered by implicit QMI driver", mm_kernel_device_get_name (port));
                     return TRUE;
                 }
             }
@@ -325,9 +339,7 @@ apply_pre_probing_filters (MMPlugin       *self,
             for (j = 0; drivers[j]; j++) {
                 /* If we match the MBIM driver: unsupported */
                 if (g_str_equal (drivers[j], "cdc_mbim")) {
-                    mm_dbg ("(%s) [%s] filtered by implicit MBIM driver",
-                            self->priv->name,
-                            mm_kernel_device_get_name (port));
+                    mm_obj_dbg (self, "port %s filtered by implicit MBIM driver", mm_kernel_device_get_name (port));
                     return TRUE;
                 }
             }
@@ -391,9 +403,7 @@ apply_pre_probing_filters (MMPlugin       *self,
           !self->priv->forbidden_product_strings) ||
          g_str_equal (mm_kernel_device_get_subsystem (port), "net") ||
          g_str_has_prefix (mm_kernel_device_get_name (port), "cdc-wdm"))) {
-        mm_dbg ("(%s) [%s] filtered by vendor/product IDs",
-                self->priv->name,
-                mm_kernel_device_get_name (port));
+        mm_obj_dbg (self, "port %s filtered by vendor/product IDs", mm_kernel_device_get_name (port));
         return TRUE;
     }
 
@@ -403,9 +413,7 @@ apply_pre_probing_filters (MMPlugin       *self,
         for (i = 0; self->priv->forbidden_product_ids[i].l; i++) {
             if (vendor == self->priv->forbidden_product_ids[i].l &&
                 product == self->priv->forbidden_product_ids[i].r) {
-                mm_dbg ("(%s) [%s] filtered by forbidden vendor/product IDs",
-                        self->priv->name,
-                        mm_kernel_device_get_name (port));
+                mm_obj_dbg (self, "port %s filtered by forbidden vendor/product IDs", mm_kernel_device_get_name (port));
                 return TRUE;
             }
         }
@@ -444,9 +452,7 @@ apply_pre_probing_filters (MMPlugin       *self,
 
         /* If we didn't match any udev tag: unsupported */
         if (!self->priv->udev_tags[i]) {
-            mm_dbg ("(%s) [%s] filtered by udev tags",
-                    self->priv->name,
-                    mm_kernel_device_get_name (port));
+            mm_obj_dbg (self, "port %s filtered by udev tags", mm_kernel_device_get_name (port));
             return TRUE;
         }
     }
@@ -493,9 +499,7 @@ apply_post_probing_filters (MMPlugin *self,
 
         if (vendor_filtered) {
             if (!self->priv->product_strings) {
-                mm_dbg ("(%s) [%s] filtered by vendor strings",
-                        self->priv->name,
-                        mm_port_probe_get_port_name (probe));
+                mm_obj_dbg (self, "port %s filtered by vendor strings", mm_port_probe_get_port_name (probe));
                 return TRUE;
             }
         } else
@@ -518,9 +522,7 @@ apply_post_probing_filters (MMPlugin *self,
         if (self->priv->product_strings) {
             /* If we didn't get any vendor or product: filtered */
             if (!vendor || !product) {
-                mm_dbg ("(%s) [%s] filtered as no vendor/product strings given",
-                        self->priv->name,
-                        mm_port_probe_get_port_name (probe));
+                mm_obj_dbg (self, "port %s filtered as no vendor/product strings given", mm_port_probe_get_port_name (probe));
                 return TRUE;
             }
             else {
@@ -541,9 +543,7 @@ apply_post_probing_filters (MMPlugin *self,
 
                 /* If we didn't match any product: unsupported */
                 if (!self->priv->product_strings[i].l) {
-                    mm_dbg ("(%s) [%s] filtered by vendor/product strings",
-                            self->priv->name,
-                            mm_port_probe_get_port_name (probe));
+                    mm_obj_dbg (self, "port %s filtered by vendor/product strings", mm_port_probe_get_port_name (probe));
                     return TRUE;
                 }
             }
@@ -563,9 +563,7 @@ apply_post_probing_filters (MMPlugin *self,
                 g_free (casefolded_product);
                 if (found) {
                     /* If we match a forbidden product: unsupported */
-                    mm_dbg ("(%s) [%s] filtered by forbidden vendor/product strings",
-                            self->priv->name,
-                            mm_port_probe_get_port_name (probe));
+                    mm_obj_dbg (self, "port %s filtered by forbidden vendor/product strings", mm_port_probe_get_port_name (probe));
                     return TRUE;
                 }
             }
@@ -579,9 +577,7 @@ apply_post_probing_filters (MMPlugin *self,
     if (self->priv->allowed_icera &&
         !mm_port_probe_is_icera (probe)) {
         /* Unsupported! */
-        mm_dbg ("(%s) [%s] filtered as modem is not icera",
-                self->priv->name,
-                mm_port_probe_get_port_name (probe));
+        mm_obj_dbg (self, "port %s filtered as modem is not icera", mm_port_probe_get_port_name (probe));
         return TRUE;
     }
 
@@ -590,9 +586,7 @@ apply_post_probing_filters (MMPlugin *self,
     if (self->priv->forbidden_icera &&
         mm_port_probe_is_icera (probe)) {
         /* Unsupported! */
-        mm_dbg ("(%s) [%s] filtered as modem is icera",
-                self->priv->name,
-                mm_port_probe_get_port_name (probe));
+        mm_obj_dbg (self, "port %s filtered as modem is icera", mm_port_probe_get_port_name (probe));
         return TRUE;
     }
 
@@ -601,9 +595,7 @@ apply_post_probing_filters (MMPlugin *self,
     if (self->priv->allowed_xmm &&
         !mm_port_probe_is_xmm (probe)) {
         /* Unsupported! */
-        mm_dbg ("(%s) [%s] filtered as modem is not XMM",
-                self->priv->name,
-                mm_port_probe_get_port_name (probe));
+        mm_obj_dbg (self, "port %s filtered as modem is not XMM", mm_port_probe_get_port_name (probe));
         return TRUE;
     }
 
@@ -612,9 +604,7 @@ apply_post_probing_filters (MMPlugin *self,
     if (self->priv->forbidden_xmm &&
         mm_port_probe_is_xmm (probe)) {
         /* Unsupported! */
-        mm_dbg ("(%s) [%s] filtered as modem is XMM",
-                self->priv->name,
-                mm_port_probe_get_port_name (probe));
+        mm_obj_dbg (self, "port %s filtered as modem is XMM", mm_port_probe_get_port_name (probe));
         return TRUE;
     }
 
@@ -775,8 +765,7 @@ mm_plugin_supports_port (MMPlugin            *self,
 
     /* Before launching any probing, check if the port is a net device. */
     if (g_str_equal (mm_kernel_device_get_subsystem (port), "net")) {
-        mm_dbg ("(%s) [%s] probing deferred until result suggested",
-                self->priv->name, mm_kernel_device_get_name (port));
+        mm_obj_dbg (self, "probing of port %s deferred until result suggested", mm_kernel_device_get_name (port));
         g_task_return_int (task, MM_PLUGIN_SUPPORTS_PORT_DEFER_UNTIL_SUGGESTED);
         g_object_unref (task);
         return;
@@ -828,10 +817,9 @@ mm_plugin_supports_port (MMPlugin            *self,
     if (self->priv->single_at &&
         mm_port_probe_list_has_at_port (mm_device_peek_port_probe_list (device)) &&
         !mm_port_probe_is_at (probe)) {
-        mm_dbg ("(%s) [%s] not setting up AT probing tasks: "
-                "modem already has the expected single AT port",
-                self->priv->name,
-                mm_kernel_device_get_name (port));
+        mm_obj_dbg (self, "not setting up AT probing tasks in port %s: "
+                    "modem already has the expected single AT port",
+                    mm_kernel_device_get_name (port));
 
         /* Assuming it won't be an AT port. We still run the probe anyway, in
          * case we need to check for other port types (e.g. QCDM) */
@@ -849,10 +837,9 @@ mm_plugin_supports_port (MMPlugin            *self,
 
     /* Launch the probe */
     probe_list_str = mm_port_probe_flag_build_string_from_mask (ctx->flags);
-    mm_dbg ("(%s) [%s] probe required: '%s'",
-            self->priv->name,
-            mm_kernel_device_get_name (port),
-            probe_list_str);
+    mm_obj_dbg (self, "probes required for port %s: '%s'",
+                mm_kernel_device_get_name (port),
+                probe_list_str);
     g_free (probe_list_str);
 
     mm_port_probe_run (probe,
@@ -932,9 +919,22 @@ mm_plugin_create_modem (MMPlugin  *self,
 
         /* Grab each port */
         for (l = port_probes; l; l = g_list_next (l)) {
-            GError *inner_error = NULL;
-            MMPortProbe *probe = MM_PORT_PROBE (l->data);
-            gboolean grabbed;
+            GError      *inner_error = NULL;
+            MMPortProbe *probe;
+            gboolean     grabbed = FALSE;
+            gboolean     force_ignored = FALSE;
+            const gchar *subsys;
+            const gchar *name;
+            const gchar *driver;
+            MMPortType   port_type;
+
+            probe = MM_PORT_PROBE (l->data);
+
+            subsys    = mm_port_probe_get_port_subsys (probe);
+            name      = mm_port_probe_get_port_name   (probe);
+            port_type = mm_port_probe_get_port_type   (probe);
+
+            driver    = mm_kernel_device_get_driver (mm_port_probe_peek_port (probe));
 
             /* If grabbing a port fails, just warn. We'll decide if the modem is
              * valid or not when all ports get organized */
@@ -943,45 +943,82 @@ mm_plugin_create_modem (MMPlugin  *self,
              * probed and accepted by the generic plugin, which is overwritten
              * by the specific one when needed. */
             if (apply_subsystem_filter (self, mm_port_probe_peek_port (probe))) {
-                grabbed = FALSE;
                 inner_error = g_error_new (MM_CORE_ERROR,
                                            MM_CORE_ERROR_UNSUPPORTED,
                                            "unsupported subsystem: '%s'",
-                                           mm_port_probe_get_port_subsys (probe));
+                                           subsys);
+                goto next;
             }
+
             /* Ports that are explicitly blacklisted will be grabbed as ignored */
-            else if (mm_port_probe_is_ignored (probe)) {
-                mm_dbg ("(%s/%s): port is blacklisted",
-                        mm_port_probe_get_port_subsys (probe),
-                        mm_port_probe_get_port_name (probe));
-                grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_peek_port (probe),
-                                                   MM_PORT_TYPE_IGNORED,
-                                                   MM_PORT_SERIAL_AT_FLAG_NONE,
-                                                   &inner_error);
+            if (mm_port_probe_is_ignored (probe)) {
+                mm_obj_dbg (self, "port %s is blacklisted", name);
+                force_ignored = TRUE;
+                goto grab_port;
             }
-#if !defined WITH_QMI
-            else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     !g_strcmp0 (mm_kernel_device_get_driver (mm_port_probe_peek_port (probe)), "qmi_wwan")) {
-                /* Try to generically grab the port, but flagged as ignored */
-                grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_peek_port (probe),
-                                                   MM_PORT_TYPE_IGNORED,
-                                                   MM_PORT_SERIAL_AT_FLAG_NONE,
-                                                   &inner_error);
+
+#if defined WITH_QMI
+            if (MM_IS_BROADBAND_MODEM_QMI (modem) &&
+                port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "qmi_wwan") != 0) {
+                /* Non-QMI net ports are ignored in QMI modems */
+                mm_obj_dbg (self, "ignoring non-QMI net port %s in QMI modem", name);
+                force_ignored = TRUE;
+                goto grab_port;
+            }
+
+            if (!MM_IS_BROADBAND_MODEM_QMI (modem) &&
+                port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "qmi_wwan") == 0) {
+                /* QMI net ports are ignored in non-QMI modems */
+                mm_obj_dbg (self, "ignoring QMI net port %s in non-QMI modem", name);
+                force_ignored = TRUE;
+                goto grab_port;
+            }
+#else
+            if (port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "qmi_wwan") == 0) {
+                /* QMI net ports are ignored if QMI support not built */
+                mm_obj_dbg (self, "ignoring QMI net port %s as QMI support isn't available", name);
+                force_ignored = TRUE;
+                goto grab_port;
             }
 #endif
-#if !defined WITH_MBIM
-            else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     !g_strcmp0 (mm_kernel_device_get_driver (mm_port_probe_peek_port (probe)), "cdc_mbim")) {
-                /* Try to generically grab the port, but flagged as ignored */
+
+#if defined WITH_MBIM
+            if (MM_IS_BROADBAND_MODEM_MBIM (modem) &&
+                port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "cdc_mbim") != 0) {
+                /* Non-MBIM net ports are ignored in MBIM modems */
+                mm_obj_dbg (self, "ignoring non-MBIM net port %s in MBIM modem", name);
+                force_ignored = TRUE;
+                goto grab_port;
+            }
+
+            if (!MM_IS_BROADBAND_MODEM_MBIM (modem) &&
+                port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "cdc_mbim") == 0) {
+                /* MBIM net ports are ignored in non-MBIM modems */
+                mm_obj_dbg (self, "ignoring MBIM net port %s in non-MBIM modem", name);
+                force_ignored = TRUE;
+                goto grab_port;
+            }
+#else
+            if (port_type == MM_PORT_TYPE_NET &&
+                g_strcmp0 (driver, "cdc_mbim") == 0) {
+                mm_obj_dbg (self, "ignoring MBIM net port %s as MBIM support isn't available", name);
+                force_ignored = TRUE;
+                goto grab_port;
+            }
+#endif
+
+        grab_port:
+            if (force_ignored)
                 grabbed = mm_base_modem_grab_port (modem,
                                                    mm_port_probe_peek_port (probe),
                                                    MM_PORT_TYPE_IGNORED,
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
-            }
-#endif
             else if (MM_PLUGIN_GET_CLASS (self)->grab_port)
                 grabbed = MM_PLUGIN_GET_CLASS (self)->grab_port (MM_PLUGIN (self),
                                                                  modem,
@@ -993,11 +1030,10 @@ mm_plugin_create_modem (MMPlugin  *self,
                                                    mm_port_probe_get_port_type (probe),
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
+
+        next:
             if (!grabbed) {
-                mm_warn ("Could not grab port (%s/%s): '%s'",
-                         mm_port_probe_get_port_subsys (MM_PORT_PROBE (l->data)),
-                         mm_port_probe_get_port_name (MM_PORT_PROBE (l->data)),
-                         inner_error ? inner_error->message : "unknown error");
+                mm_obj_warn (self, "could not grab port %s: %s", name, inner_error ? inner_error->message : "unknown error");
                 g_clear_error (&inner_error);
             }
         }
@@ -1019,18 +1055,18 @@ mm_plugin_create_modem (MMPlugin  *self,
              * installed yet). */
             kernel_device = mm_kernel_device_generic_new_with_rules (properties, NULL, &inner_error);
             if (!kernel_device) {
-                mm_warn ("Could not grab port (virtual/%s): '%s'",
-                         virtual_ports[i],
-                         inner_error ? inner_error->message : "unknown error");
+                mm_obj_warn (self, "could not create generic device for virtual port %s: %s",
+                             virtual_ports[i],
+                             inner_error ? inner_error->message : "unknown error");
                 g_clear_error (&inner_error);
             } else if (!mm_base_modem_grab_port (modem,
                                                  kernel_device,
                                                  MM_PORT_TYPE_AT,
                                                  MM_PORT_SERIAL_AT_FLAG_NONE,
                                                  &inner_error)) {
-                mm_warn ("Could not grab port (virtual/%s): '%s'",
-                         virtual_ports[i],
-                         inner_error ? inner_error->message : "unknown error");
+                mm_obj_warn (self, "could not grab virtual port %s: %s",
+                             virtual_ports[i],
+                             inner_error ? inner_error->message : "unknown error");
                 g_clear_error (&inner_error);
             }
 
@@ -1049,6 +1085,19 @@ mm_plugin_create_modem (MMPlugin  *self,
 
 /*****************************************************************************/
 
+static gchar *
+log_object_build_id (MMLogObject *_self)
+{
+    MMPlugin         *self;
+    g_autofree gchar *plugin_name_lowercase;
+
+    self = MM_PLUGIN (_self);
+    plugin_name_lowercase = g_ascii_strdown (self->priv->name, -1);
+    return g_strdup_printf ("plugin/%s", plugin_name_lowercase);
+}
+
+/*****************************************************************************/
+
 static void
 mm_plugin_init (MMPlugin *self)
 {
@@ -1075,6 +1124,10 @@ set_property (GObject *object,
         /* Construct only */
         self->priv->name = g_value_dup_string (value);
         break;
+    case PROP_IS_GENERIC:
+        /* Construct only */
+        self->priv->is_generic = g_value_get_boolean (value);
+        break;
     case PROP_ALLOWED_SUBSYSTEMS:
         /* Construct only */
         self->priv->subsystems = g_value_dup_boxed (value);
@@ -1197,6 +1250,9 @@ get_property (GObject *object,
     case PROP_NAME:
         g_value_set_string (value, self->priv->name);
         break;
+    case PROP_IS_GENERIC:
+        g_value_set_boolean (value, self->priv->is_generic);
+        break;
     case PROP_ALLOWED_SUBSYSTEMS:
         g_value_set_boxed (value, self->priv->subsystems);
         break;
@@ -1308,6 +1364,12 @@ finalize (GObject *object)
     G_OBJECT_CLASS (mm_plugin_parent_class)->finalize (object);
 }
 
+static void
+log_object_iface_init (MMLogObjectInterface *iface)
+{
+    iface->build_id = log_object_build_id;
+}
+
 static void
 mm_plugin_class_init (MMPluginClass *klass)
 {
@@ -1328,6 +1390,14 @@ mm_plugin_class_init (MMPluginClass *klass)
                               NULL,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property
+        (object_class, PROP_IS_GENERIC,
+         g_param_spec_boolean (MM_PLUGIN_IS_GENERIC,
+                               "Generic",
+                               "Whether the plugin is the generic one",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_class_install_property
         (object_class, PROP_ALLOWED_SUBSYSTEMS,
          g_param_spec_boxed (MM_PLUGIN_ALLOWED_SUBSYSTEMS,
